@@ -1,16 +1,31 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+import re
 import json
+import os
 from datetime import datetime
 from itertools import imap, repeat
 from functools import wraps
 from HTMLParser import HTMLParser
+from tempfile import mkdtemp
 
 import requests
+from PIL import Image
 
 _BASE_URL = "http://a.4cdn.org/"
 
 
+def memoize(func):
+    cache = {}
+
+    @wraps(func)
+    def wrapper(*args):
+        if args not in cache:
+            cache[args] = func(*args)
+        return cache[args]
+    return wrapper
+
+
 def unescape_html(string):
     return HTMLParser().unescape(string)
 
@@ -108,8 +123,12 @@ class Thread(ImageBoardItem):
         self._items = tuple(
             imap(Post, posts, repeat(board), repeat(self))
         )
+
+        self._post_table = {p.no: p for p in self._items}
         self._lastfetch = datetime.utcnow()
 
+        self.tmpdir = mkdtemp(prefix='')
+
     @classmethod
     def from_id(cls, board, id):
         thread = requests.get(cls.url.format(board, id))
@@ -154,6 +173,7 @@ class Thread(ImageBoardItem):
             self._items = tuple(
                 imap(Post, posts, repeat(self.board), repeat(self))
             )
+            self._post_table.update({p.no: p for p in self._items})
         elif res.status_code == 304:
             return
         else:
@@ -161,22 +181,53 @@ class Thread(ImageBoardItem):
 
 
 class Post(object):
+    _greentext = r"""class="quote">>(?P<greentext>.*)</span>"""
+    _link = r"""class="quotelink">>>(?P<greentext>\d+)</a>"""
+
     def __init__(self, post, board, thread):
         self.board = board
         self._thread = thread
+
+        self._assign_fields(post)
+
+        self._lastfetch = datetime.utcnow()
+
+    def __repr__(self):
+        return "<Post> {p.no}, created at {p.now} by {p.name}".format(p=self)
+
+    def _assign_fields(self, post):
         for k, v in post.iteritems():
             if isinstance(v, unicode):
                 v = unescape_html(v)
             setattr(self, k, v)
 
-        self._lastfetch = datetime.utcnow()
+    @property
+    def greentext(self):
+        greentext = re.findall(self._greentext, getattr(self, 'com', ''))
+        return tuple(imap(unicode.strip, greentext))
 
-    def __repr__(self):
-        return "<Post> {op.no}, created at {op.now} by {op.name}"
+    @property
+    def links(self):
+        links = []
+        for id in imap(int, re.findall(self._link, self.com)):
+            links.append(self._thread._post_table[id])
+        return tuple(links)
 
     def _fetch(self):
         self._thread.fetch()
 
-    @update()
-    def __getattr__(self, val):
-        return getattr(self, val)
+    @property
+    def image(self):
+        img = 'http://i.4cdn.org/{p.board}/{p.tim}{p.ext}'.format(p=self)
+        filename = '{p.tim}{p.ext}'.format(p=self)
+        return Post._image(os.path.join(self._thread.tmpdir, filename), img)
+
+    @staticmethod
+    @memoize
+    def _image(filename, img):
+        with open(filename, 'wb') as f:
+            f.write(requests.get(img).content)
+
+        img = Image.open(filename)
+        os.remove(filename)
+        return img
